/***********************************************************************/
/*                                                                     */
/*  FILE        :ICS_sample.c                                          */
/*  DATE        :Oct, 8, 2018                                          */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :RX72T                                                 */
/*                                                                     */
/*  NOTE:THIS IS AN ICS++ EXAMPLE.                                     */
/*                                                                     */
/***********************************************************************/

#ifdef  __ICCRX__
  #include "r_cg_macrodriver.h"
  #include <math.h>
#elif   __CCRX__
  #include <machine.h>
  #include <mathf.h>
  #include <stdint.h>
  #include "iodefine.h"
#else
  //
#endif

#include "ICS2_RX72T.h"

void main(void);

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Set option bytes */

#ifdef   __ICCRX__

  #if __LITTLE_ENDIAN__
    #pragma public_equ="__MDES",0xFFFFFFFFUL
  #else
    #pragma public_equ="__MDES",0xFFFFFFF8UL
  #endif

  #pragma public_equ="__OFS0",0xFFFFFFFFUL
  #pragma public_equ="__OFS1",0xFFFFFFFFUL

#elif   __CCRX__  
  
  
#endif


#define   TWOPI    (2.0f*3.1415926535f)
#define   BRRB  0      // if HOCO=20MHz PLL=x10 = PCLKB=50MHz,  ICS=6.25Mbps, DTLScope CLK=50MHz
#define   BRRA  1      // if HOCO=20MHz PLL=x10 = PCLKA=100MHz, ICS=6.25Mbps, DTLScope CLK=50MHz
                       //  PCLK  BRR  BPS   DTLclock 
                       //   60M   0  7.500M   60MHz  
                       //   60M   1  3.750M   30MHz  
                       //   60M   2  2.500M   20MHz
                       //   60M   3  1.875M   15MHz    
                       //   60M   4  1.500M   12MHz
                       //   60M  251 1.000M    8MHz

                       //   50M   0  6.250M   50MHz  
                       //   50M   1  3.125M   25MHz  
                       //   50M   2  2.083M   16.7MHz
                       //   50M   3  1.563M   12.5MHz    
                       //   50M   4  1.250M   10MHz
                       //   50M  250 1.000M    8MHz

                       //   40M   0  5.000M   40MHz  
                       //   40M   1  2.500M   20MHz  
                       //   40M   2  1.667M   13.333MHz
                       //   40M   3  1.250M   10MHz    
                       //   40M   4  1.000M    8MHz

#define   MODE   3     // 0:16bit 8ch,one time (Can not use)
                       // 1:32bit 8ch,2 times
                       // 2:32bit 4ch,one time
                       // 3:32bit 12ch 3 times


#ifdef  __ICCRX__
  #pragma location=0x0F000
  __no_init  uint32_t dtc_table[256]; /* alignment 0x000 */
#elif   __CCRX__
  #pragma address dtc_table=0x0F000
  uint32_t dtc_table[256];            /* alignment 0x000 */
#else
#endif

float   wt    = 0.0f;
float   dwt   = TWOPI*50.0f/10000.0f;
float   refu  = 0.0f;
float   refv  = 0.0f;
float   refw  = 0.0f;
float   m     = 1.0f;
float   dm    = 0.0002;
int32_t up    = 0;


void main(void)
{
    SYSTEM.PRCR.WORD = 0xA50B;
    SYSTEM.OSTDCR.BYTE = 0x00;

    SYSTEM.HOCOCR2.BYTE = 0x02;      // HOCO = 20MHz
    SYSTEM.HOCOCR.BYTE  = 0x00;      // HOCO start
    while (SYSTEM.OSCOVFSR.BIT.HCOVF==0) { /* no code */ }  // Wait until HOCO stable
    
    SYSTEM.SCKCR3.WORD = 0x0100;     // Set system clock = HOCO
  #if 1  // CLK=200MHz
    SYSTEM.SCKCR.LONG  = 0x20821222; // FCK=1/4, ICK=1/1, BCLK output stop, BCK=1/4
                                     // FCK  =1/4 =  50MHz            (Max 60MHz)
                                     // ICK  =1/1 = 200MHz            (Max200MHz)
                                     // PCLKA=1/2 = 100MHz, SCIF,MTU,GPT (Max120MHz)
                                     // PCLKB=1/4 =  50MHz, others    (Max 60MHz)
                                     // PCLKC=1/4 =  50MHz, ADC unit0 (Max160MHz)
                                     // PCLKD=1/4 =  50MHz, ADC unit1 (Max 60MHz)
    SYSTEM.MEMWAIT.BYTE = 0x01;      // Force 1 wait for higher clock
    SYSTEM.PLLCR.BIT.PLIDIV    = 0;  // PLL input is divided by 1
    SYSTEM.PLLCR.BIT.PLLSRCSEL = 1;  // PLL source is HOCO
    SYSTEM.PLLCR.BIT.STC       = 19; // PLL mul rate is x10
    SYSTEM.PLLCR2.BYTE         = 0;  // PLL on
  #endif
  #if 0 // CLK=120MHz
    SYSTEM.SCKCR.LONG  = 0x21821222; // FCK=1/4, ICK=1/2, BCLK output stop, BCK=1/4
                                     // FCK  =1/4 =  60MHz            (Max 60MHz)
                                     // ICK  =1/2 = 120MHz            (Max200MHz) ****
                                     // PCLKA=1/2 = 120MHz, SCIF,MTU,GPT (Max120MHz)
                                     // PCLKB=1/4 =  60MHz, others    (Max 60MHz)
                                     // PCLKC=1/4 =  60MHz, ADC unit0 (Max160MHz)
                                     // PCLKD=1/4 =  60MHz, ADC unit1 (Max 60MHz)
    SYSTEM.MEMWAIT.BYTE = 0x01;      // Force 1 wait for higher clock
    SYSTEM.PLLCR.BIT.PLIDIV    = 0;  // PLL input is divided by 1
    SYSTEM.PLLCR.BIT.PLLSRCSEL = 1;  // PLL source is HOCO
    SYSTEM.PLLCR.BIT.STC       = 23; // PLL mul rate is x12 (20x12=240MHz)
    SYSTEM.PLLCR2.BYTE         = 0;  // PLL on
  #endif
  #if 0 // CLK=240MHz over clock
    SYSTEM.SCKCR.LONG  = 0x20821222; // FCK=1/4, ICK=1/1, BCLK output stop, BCK=1/4
                                     // FCK  =1/4 =  60MHz            (Max 60MHz)
                                     // ICK  =1/1 = 240MHz            (Max200MHz) ****
                                     // PCLKA=1/2 = 120MHz, SCIF,MTU,GPT (Max120MHz)
                                     // PCLKB=1/4 =  60MHz, others    (Max 60MHz)
                                     // PCLKC=1/4 =  60MHz, ADC unit0 (Max160MHz)
                                     // PCLKD=1/4 =  60MHz, ADC unit1 (Max 60MHz)
    SYSTEM.MEMWAIT.BYTE = 0x01;      // Force 1 wait for higher clock
    SYSTEM.PLLCR.BIT.PLIDIV    = 0;  // PLL input is divided by 1
    SYSTEM.PLLCR.BIT.PLLSRCSEL = 1;  // PLL source is HOCO
    SYSTEM.PLLCR.BIT.STC       = 23; // PLL mul rate is x12
    SYSTEM.PLLCR2.BYTE         = 0;  // PLL on

  #endif
    while (SYSTEM.OSCOVFSR.BIT.PLOVF==0) { /* no code */ } // Wait until PLL stable
    
    SYSTEM.SCKCR3.WORD = 0x0400;     // Set system clock = PLL (20MHzx10=200MHz)

    FLASH.ROMCE.WORD = 1;

  //******************************************************************************************************
                                                                     // RENESAS   DTL
                                                                     //CPU Card  T6214
  //PCLKB=50MHz  CPUCLK=200MHz                                        // 144pin  144pin
    ics2_init((void *)(&dtc_table), ICS_SCI6_PB0_PB1,  6, 250, MODE); //  CN6     CN13  OK (1.00Mbps)
  //PCLKB=60MHz  CPUCLK=120MHz
  //ics2_init((void *)(&dtc_table), ICS_SCI6_PB0_PB1,  6, 251, MODE); //  CN6     CN13  OK (1.00Mbps)

  //ics2_init((void *)(&dtc_table), ICS_SCI1_PD3_PD5,  6, BRRB, MODE); //         CN15  OK (E1)
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PD7_PE0,  6, BRRB, MODE); //  --     CN20  OK (E1)
  //ics2_init((void *)(&dtc_table), ICS_SCI5_PB5_PB6,  6, BRRB, MODE); //  CN2    CN18  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI6_PB0_PB1,  6, BRRB, MODE); //  CN6    CN13  OK (default)

  //ics2_init((void *)(&dtc_table), ICS_SCI8_PD0_PD1,  6, BRRB, MODE); //  --     CN16  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI8_PC1_PC0,  6, BRRB, MODE); //  CN4    CN17  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI8_PA4_PA5,  6, BRRB, MODE); //  --     CN11  OK (Z)
  //ics2_init((void *)(&dtc_table), ICS_SCI8_P23_P22,  6, BRRB, MODE); //  --     CN5   OK (HALL) U
  //ics2_init((void *)(&dtc_table), ICS_SCI9_P01_P00,  6, BRRB, MODE); //  --     CN14  OK (E1/FO2) R33
  //ics2_init((void *)(&dtc_table), ICS_SCI9_PA3_PA2,  6, BRRB, MODE); //  --     CN10  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI9_PG1_PG0,  6, BRRB, MODE); //  --     CN7   OK (PWM)
  //ics2_init((void *)(&dtc_table), ICS_SCI11_PD3_PD5, 6, BRRA, MODE); //  --     CN15  OK (E1)
  //ics2_init((void *)(&dtc_table), ICS_SCI11_PB5_PB6, 6, BRRA, MODE); //  CN6    CN18  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI11_PF0_PF1, 6, BRRA, MODE); //  CN6    CN19    
  //ics2_init((void *)(&dtc_table), ICS_SCI12_PB5_PB6, 6, BRRB, MODE); //  CN6    CN18  OK
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P23_P22, 6, BRRB, MODE); //  --     CN5   OK (HALL)
  //ics2_init((void *)(&dtc_table), ICS_SCI12_P01_P00, 6, BRRB, MODE); //  --     CN14  OK (E1/FO2) R33

  //******************************************************************************************************

#ifdef  __ICCRX__
    EI();
#elif   __CCRX__
    set_ipl(0); /* set lowest IPL level */
    setpsw_i();  // enable interrput
#else
    //
#endif

    while (1)
    {
        int32_t i;
        
        wt = wt + dwt;
        if (wt>TWOPI) { wt = wt - TWOPI; }
        
        if (up==1)
        {
             m = m + dm;
             if (m>=1.0f) { up = 0; }
        }
        else
        {
             m = m - dm;
             if (m<=0.5f) { up = 1; }
        }

        refu  = m * sinf(wt);
        refv  = m * sinf(wt-TWOPI*1.0f/3.0f);
        refw  = m * sinf(wt-TWOPI*2.0f/3.0f);

        for(i=0;i<20000;i++) { nop(); }

        ics2_watchpoint();
    }
}
